@@ -160,7 +160,7 @@ string g_cachedir("/var/cache/upmpdcli");
 string g_configfilename;
 ConfSimple *g_config;
 ConfSimple *g_state;
-bool g_enableL16 = false;
+bool g_enableL16 = true;
 bool g_lumincompat = false;
 
 static void onsig(int)
@@ -650,14 +650,13 @@ int main(int argc, char *argv[])
         // Only the "special" upmpdcli 0.19.16 version has patch != 0
         g_enableL16 = mpdstat.versmajor >= 1 || mpdstat.versminor >= 20 ||
             mpdstat.verspatch >= 16;
-        // Also L16 is a major source of issues when playing with
-        // win10 'cast to device', inciting it to transcode for some
-        // reason, with very bad results. So for the future (new in
-        // 1.5), only enable it if it's explicitely required by the
-        // config.
-        bool confl16{false};
+        // It appeared in the past that L16 is a major source of issues when
+        // playing with win10 'cast to device', inciting it to transcode for
+        // some reason, with very bad results. Can't reproduce this now. So
+        // change config default to true.
+        bool confl16{true};
         if (g_config) {
-            confl16 = g_config->getBool("enablel16", false);
+            confl16 = g_config->getBool("enablel16", true);
         }
         g_enableL16 = g_enableL16 && confl16;
     }
