@@ -35,7 +35,6 @@ import subprocess
 import pwd
 import errno
 
-PY3 = sys.version > '3'
 import conftree
 
 default_mime = "audio/mpeg"
@@ -192,7 +191,9 @@ def getserviceuserpass(upconfig, servicename):
     username = upconfig.get(servicename + 'user')
     password = upconfig.get(servicename + 'pass')
     if not username or not password:
-        altconf = conftree.ConfSimple('/var/cache/upmpdcli/ohcreds/screds')
+        credsfile = os.path.join(getcachedir(upconfig, ''), 'ohcreds', 'screds')
+        uplog("Retrieving user/pass from %s" % credsfile)
+        altconf = conftree.ConfSimple(credsfile)
         username = altconf.get(servicename + 'user')
         password = altconf.get(servicename + 'pass')
     return username, password
@@ -200,10 +201,7 @@ def getserviceuserpass(upconfig, servicename):
 def uplog(s):
     if not type(s) == type(b''):
         s = ("%s: %s" % (_idprefix, s)).encode('utf-8')
-    if PY3:
         sys.stderr.buffer.write(s + b'\n')
-    else:
-        sys.stderr.write(s + b'\n')
     sys.stderr.flush()
 
 
@@ -230,9 +228,7 @@ def getcachedir(config, servicename):
 # difficult. Executing "ip addr" actually seems to be the simplest
 # approach, only works on Linux though (maybe bsd too ?)
 def findmyip():
-    data = subprocess.check_output(["ip", "addr"])
-    if PY3:
-        data = data.decode('utf-8')
+    data = subprocess.check_output(["ip", "addr"]).decode('utf-8')
     l = data.split()
     ips = []
     chosenip = ""
