@@ -24,14 +24,18 @@
 #
 # The _dirvec list has one entry for each directory. Directories are
 # created as needed by splitting the paths/urls from _rcldocs (and
-# possibly adding some for contentgroups). Directories have no
-# direct relation with the index objects, they are identified by their
-# _dirvec index
+# possibly adding some for groupings defined by the Group
+# tag). Directories have no direct relation with the index objects,
+# they are identified by their _dirvec index
 #
 # Obect ids inside the section:
 #    Container: $d<diridx> where <diridx> indexes into _dirvec
 #    Item: $i<docidx> where <docidx> indexes into _rcldocs
 #
+# Note: this is very different from what Minim does. Minim uses actual
+#   objid paths as objids. E.g. 0$folders$f1589$f1593$f1604$*i11609
+#   Must make pwd and any walk up the tree much easier.
+#
 # Each _dirvec entry is a Python dict, mapping the directory entries'
 # names to a pair (diridx,docidx), where:
 #
@@ -40,7 +44,7 @@
 #     - There is no _rcldocs entry, which could possibly happen if
 #       there is no result for an intermediary element in a path,
 #       because of some recoll issue, or because this is a synthetic
-#       'contentgroup' entry.
+#       'Group' entry.
 #     - Or, while we build the structure, temporarily, if the doc was
 #       not yet seen. The value will then be updated when we see it.
 #
@@ -48,6 +52,10 @@
 # think that it is ever used. The Recoll doc for a directory has
 # nothing very interesting in it.
 #
+# Note: We could probably use a single value, with a convention
+# saying, e.g., that > 0 is for docs and < -1 for folders. Check if
+# this saves a significant amount of memory.
+#
 # Each directory has a special ".." entry with a diridx pointing to
 # the parent directory. This allows building a path from a container
 # id (aka pwd).
@@ -69,23 +77,37 @@
 import os
 import shlex
 import sys
-PY3 = sys.version > '3'
-if PY3:
-    from urllib.parse import quote as urlquote
-else:
-    from urllib import quote as urlquote
-
+from urllib.parse import quote as urlquote
 import time
 from timeit import default_timer as timer
 
-from upmplgutils import uplog
-from uprclutils import docarturi, audiomtypes, rcldirentry, \
-     rcldoctoentry, cmpentries
+from upmplgutils import uplog, direntry
+from uprclutils import audiomtypes, rcldoctoentry, cmpentries
 import uprclutils
 from recoll import recoll
+try:
+    from recoll import qresultstore
+    _has_resultstore = True
+except:
+    _has_resultstore = False
+uprclutils.sethasresultstore(_has_resultstore)
+
 from recoll import rclconfig
 import uprclinit
 
+# All Doc fields which we may want to access (reserve slots in the
+# resultstore). We use an inclusion list, and end up with a smaller
+# store than by using an exclusion list, but it's a bit more difficult
+# to manage.
+#
+# +  possibly 'xdocid' and/or 'rcludi' if/when needed?
+#
+_otherneededfields = [
+    'albumartist', 'allartists', 'comment', 'composer', 'conductor',
+    'contentgroup', 'date', 'dmtime', 'discnumber', 'embdimg', 'filename',
+    'genre', 'group', 'label', 'lyricist', 'orchestra', 'performer',
+]
+
 class Folders(object):
 
     # Initialize (read recoll data and build tree).
@@ -95,11 +117,14 @@ class Folders(object):
         self._pprefix = pathprefix
         # Debug : limit processed recoll entries for speed
         self._maxrclcnt = 0
+        # Overflow storage for synthetic records created for playlists
+        # url entries. Uses docidx values starting at len(_rcldocs),
+        # with actual index value - len(_rcldocs)
+        self._moredocs = []
         self._fetchalldocs(confdir)
         self._rcl2folders(confdir)
-        self._enabletags = uprclinit.g_minimconfig.getboolvalue("showExtras",
-                                                                True)
-
+        self._enabletags = \
+            uprclinit.g_minimconfig.getboolvalue("showExtras", True)
 
     def rcldocs(self):
         return self._rcldocs
@@ -126,14 +151,41 @@ class Folders(object):
         return myidx
     
 
+    # Find the doc index for a filesystem path.
+    # We use a temporary doc to call _stat()
+    def statpath(self, plpath, path):
+        if not os.path.isabs(path):
+            path = os.path.join(os.path.dirname(plpath), path)
+        doc = recoll.Doc()
+        if _has_resultstore:
+            doc["url"] = b'file://' + path
+        else:
+            doc.setbinurl(bytearray(b'file://' + path))
+        fathidx, docidx = self._stat(doc)
+        if docidx >= 0 and docidx < len(self._rcldocs):
+            return docidx
+        uplog("No track found for playlist %s entry %s" % (plpath, path))
+        return None
+
+
+    # Create bogus doc for external (http) url. This is for playlists.
+    def docforurl(self, url):
+        doc = recoll.Doc()
+        doc.url = url
+        elt = os.path.split(url)[1]
+        tt = elt.decode('utf-8', errors='ignore')
+        doc.title = tt
+        # Temp workaround for recoll 1.28.2 not setting values in meta
+        doc.text = tt
+        doc.mtype = "audio/mpeg"
+        return doc
+    
+
     # Initialize all playlists after the tree is otherwise complete
     def _initplaylists(self):
         for diridx in self._playlists:
             pldocidx = self._dirvec[diridx]["."][1]
             pldoc = self._rcldocs[pldocidx]
-            arturi = uprclutils.docarturi(pldoc, self._httphp,self._pprefix)
-            if arturi:
-                pldoc.albumarturi = arturi
             plpath = uprclutils.docpath(pldoc)
             try:
                 m3u = uprclutils.M3u(plpath)
@@ -143,27 +195,23 @@ class Folders(object):
             for url in m3u:
                 if m3u.urlRE.match(url):
                     # Actual URL (usually http). Create bogus doc
-                    doc = recoll.Doc()
-                    doc.setbinurl(bytearray(url))
-                    elt = os.path.split(url)[1]
-                    doc.title = elt.decode('utf-8', errors='ignore')
-                    doc.mtype = "audio/mpeg"
-                    self._rcldocs.append(doc)
-                    docidx = len(self._rcldocs) -1
-                    self._dirvec[diridx][elt] = (-1, docidx)
+                    doc = self.docforurl(url)
+                    self._moredocs.append(doc)
+                    docidx = len(self._rcldocs) + len(self._moredocs) - 1
+                    tt = doc["title"]
+                    if not tt:
+                        # Temp workaround for recoll 1.28.2 not
+                        # setting values in meta
+                        tt = doc["text"]
+                    self._dirvec[diridx][tt] = (-1, docidx)
                 else:
-                    doc = recoll.Doc()
-                    doc.setbinurl(bytearray(b'file://' + url))
-                    fathidx, docidx = self._stat(doc)
-                    if docidx >= 0 and docidx < len(self._rcldocs):
+                    docidx = self.statpath(plpath, url)
+                    if docidx:
                         elt = os.path.split(url)[1]
                         self._dirvec[diridx][elt] = (-1, docidx)
-                    else:
-                        #uplog("No track found: playlist %s entry %s" %
-                        #      (plpath,url))
-                        pass
         del self._playlists
 
+
     # The root entry (diridx 0) is special because its keys are the
     # topdirs paths, not simple names. We look with what topdir path
     # this doc belongs to, then return the appropriate diridx and the
@@ -171,15 +219,15 @@ class Folders(object):
     def _pathbeyondtopdirs(self, doc):
         url = uprclutils.docpath(doc).decode('utf-8', errors='replace')
         # Determine the root entry (topdirs element). Special because
-        # its path is not a simple name.
-        fathidx = -1
+        # its path is not a simple name. Fathidx is its index in _dirvec
+        firstdiridx = -1
         for rtpath,idx in self._dirvec[0].items():
             #uplog("type(url) %s type(rtpath) %s rtpath %s url %s" %
             # (type(url),type(rtpath),rtpath, url))
             if url.startswith(rtpath):
-                fathidx = idx[0]
+                firstdiridx = idx[0]
                 break
-        if fathidx == -1:
+        if firstdiridx == -1:
             # uplog("No parent in topdirs: %s" % url)
             return None,None
 
@@ -188,18 +236,18 @@ class Folders(object):
         if len(url1) == 0:
             return None,None
 
-        # If there is a contentgroup field, just add it as a virtual
+        # If there is a Group field, just add it as a virtual
         # directory in the path. This only affects the visible tree,
         # not the 'real' URLs of course.
-        if doc.contentgroup:
+        if doc["group"]:
             a = os.path.dirname(url1)
             b = os.path.basename(url1)
-            url1 = os.path.join(a, doc.contentgroup, b)
+            url1 = os.path.join(a, doc["group"], b)
             
         # Split path. The caller will walk the list (possibly creating
         # directory entries as needed, or doing something else).
         path = url1.split('/')[1:]
-        return fathidx, path
+        return firstdiridx, path
     
 
     # Main folders build method: walk the recoll docs array and split
@@ -245,21 +293,15 @@ class Folders(object):
             
             # Only include selected mtypes: tracks, playlists,
             # directories etc.
-            if doc.mtype not in audiomtypes:
+            if doc["mtype"] not in audiomtypes:
                 continue
 
             # For linking item search results to the main
             # array. Deactivated for now as it does not seem to be
-            # needed.
-            #self._xid2idx[doc.xdocid] = docidx
+            # needed (and we would need to add xdocid to the
+            # resultstore fields).
+            #self._xid2idx[doc["xdocid"]] = docidx
             
-            # Possibly enrich the doc entry with a cover art uri.
-            arturi = docarturi(doc, self._httphp, self._pprefix)
-            if arturi:
-                # The uri is quoted, so it's ascii and we can just store
-                # it as a doc attribute
-                doc.albumarturi = arturi
-
             fathidx, path = self._pathbeyondtopdirs(doc)
             if not fathidx:
                 continue
@@ -286,9 +328,9 @@ class Folders(object):
                         fathidx = self._createdir(fathidx, -1, elt)
                     else:
                         # Last element. If directory, needs a self._dirvec entry
-                        if doc.mtype == 'inode/directory':
+                        if doc["mtype"] == 'inode/directory':
                             fathidx = self._createdir(fathidx, docidx, elt)
-                        elif doc.mtype == 'audio/x-mpegurl':
+                        elif doc["mtype"] == 'audio/x-mpegurl':
                             fathidx = self._createpldir(fathidx, docidx,doc,elt)
                         else:
                             self._dirvec[fathidx][elt] = (-1, docidx)
@@ -297,18 +339,19 @@ class Folders(object):
             for ent in self._dirvec:
                 uplog("%s" % ent)
 
-
         self._initplaylists()
                     
         end = timer()
         uplog("_rcl2folders took %.2f Seconds" % (end - start))
 
+
     # Fetch all the docs by querying Recoll with [mime:*], which is
     # guaranteed to match every doc without overflowing the query size
     # (because the number of mime types is limited). Something like
     # title:* would overflow. This creates the main doc array, which is
     # then used by all modules.
     def _fetchalldocs(self, confdir):
+        #uplog("_fetchalldocs: has_resultstore: %s" % _has_resultstore)
         start = timer()
 
         rcldb = recoll.connect(confdir=confdir)
@@ -317,24 +360,32 @@ class Folders(object):
         #rclq.execute('album:a* OR album:b* OR album:c*', stemming=0)
         uplog("Estimated alldocs query results: %d" % (rclq.rowcount))
 
-        tagaliases = None
-        if uprclinit.g_minimconfig:
-            tagaliases = uprclinit.g_minimconfig.gettagaliases()
+        if _has_resultstore:
+            fields = [r[1] for r in uprclutils.upnp2rclfields.items()]
+            fields += _otherneededfields
+            fields = list(set(fields))
+            #uplog("_fetchalldocs: store fields: %s" % fields)
+            self._rcldocs = qresultstore.QResultStore()
+            self._rcldocs.storeQuery(rclq, fieldspec=fields, isinc = True)
+        else:
+            tagaliases = None
+            if uprclinit.g_minimconfig:
+                tagaliases = uprclinit.g_minimconfig.getaliastags()
+            self._rcldocs = []
+            for doc in rclq:
+                if tagaliases:
+                    for orig, target, rep in tagaliases:
+                        val = doc[orig]
+                        #uplog("Rep %s doc[%s]=[%s] doc[%s]=[%s]"%
+                        #      (rep, orig, val, target, doc[target]))
+                        if val and (rep or not doc[target]):
+                            setattr(doc, target, val)
 
-        self._rcldocs = []
-        for doc in rclq:
-            if tagaliases:
-                for orig, target, rep in tagaliases:
-                    val = doc.get(orig)
-                    #uplog("Rep %s doc[%s]=[%s] doc[%s]=[%s]"%
-                    #      (rep, orig, val, target, doc.get(target)))
-                    if val and (rep or not doc.get(target)):
-                        setattr(doc, target, val)
+                self._rcldocs.append(doc)
+                if self._maxrclcnt > 0 and len(self._rcldocs) >= self._maxrclcnt:
+                    break
+                time.sleep(0)
 
-            self._rcldocs.append(doc)
-            if self._maxrclcnt > 0 and len(self._rcldocs) >= self._maxrclcnt:
-                break
-            time.sleep(0)
         end = timer()
         uplog("Retrieved %d docs in %.2f Seconds" %
               (len(self._rcldocs), end - start))
@@ -358,8 +409,7 @@ class Folders(object):
             pathremain = ''
         else:
             if dirpth[0:2] != '$d':
-                raise Exception("folders:browse: called on non dir objid %s" %
-                                pid)
+                raise Exception("folders:browse: called on non dir objid %s" % pid)
             # Other $sign?
             nextdol = dirpth.find("$", 1)
             if nextdol > 0:
@@ -378,33 +428,45 @@ class Folders(object):
 
     # Tell the top module what entries we define in the root
     def rootentries(self, pid):
-        return [rcldirentry(pid + 'folders', pid, '[folders]'),]
+        return [direntry(pid + 'folders', pid, '[folders]'),]
 
 
     # Look all non-directory docs inside directory, and return the
-    # cover art we find. The doc albumarturi field has been set during
-    # the initial walk by call to uprclutils.docarturi()
+    # cover art we find. 
     #
-    # TBD In the case where this is a contentgroup directory, we'd
-    # need to go look into the file system for a group.xxx
-    # image. Actually, the best approach would probably be to create
-    # virtual doc records for such directories, and set their
-    # albumarturi during the tree setup. As it is things work if the
-    # tracks rely on the group pic (instead of having an embedded pic
-    # or track pic) Also: playlists: need to look at the physical dir
-    # for a e.g. playlistname.jpg. 
+    # TBD In the case where this is a Group directory, we'd
+    # need to go look into the file system for a group.xxx image.  As
+    # it is, things work if the tracks rely on the group pic (instead
+    # of having an embedded pic or track pic) Also: playlists: need to
+    # look at the physical dir for a e.g. playlistname.jpg.  And also:
+    # currently, we won't look at possible art for a folder with no
+    # music (e.g. top folder of a multi-cd). We'd need to look at the
+    # fs directory, but we don't have the path at this point. We'd
+    # need a '.' entry with a doc record containing the
+    # path. Currently this works if one of the subdirs has an audio
+    # file with an external cover.
     def _arturifordir(self, diridx):
         for nm,ids in self._dirvec[diridx].items():
-            if ids[1] >= 0:
-                doc = self._rcldocs[ids[1]]
-                if doc.mtype != 'inode/directory' and doc.albumarturi:
-                    return doc.albumarturi
+            docidx = ids[1]
+            if docidx >= 0 and docidx < len(self._rcldocs):
+                doc = self._rcldocs[docidx]
+                # We used to only look for art for direct children
+                # tracks, but we now also look at subdirs. This will
+                # yield an image from the first subdir which has an
+                # image file in it, so somewhat random, but nice
+                # anyway. The condition is kept around to show how to
+                # change our minds or make it optional.
+                if True or doc["mtype"] != 'inode/directory':
+                    arturi = uprclutils.docarturi(
+                        doc, self._httphp, self._pprefix)
+                    if arturi:
+                        return arturi
               
 
     # Folder hierarchy browse method.
     # objid is like folders$index
     # flag is meta or children.
-    def browse(self, pid, flag):
+    def browse(self, pid, flag, offset, count):
         
         diridx,pthremain = self._objidtodiridx(pid)
 
@@ -413,8 +475,8 @@ class Folders(object):
         if pthremain:
             if not pthremain.find('$tagview.0') == 0:
                 raise Exception("uprclfolders: pid [%s]. bad pthremain"%pid)
-            return uprclinit.g_trees['tags'].browseFolder(pid, flag, pthremain,
-                                                          self.dirpath(pid))
+            return uprclinit.getTree('tags').browseFolder(pid, flag, pthremain,
+                                                             self.dirpath(pid))
         
         # If there is only one entry in root, skip it. This means that 0
         # and 1 point to the same dir, but this does not seem to be an
@@ -423,7 +485,7 @@ class Folders(object):
             diridx = 1
         
         #uplog("Folders browse: diridx %d content: [%s]" %
-        #    (diridx,self._dirvec[diridx]))
+        #      (diridx,self._dirvec[diridx]))
 
         entries = []
         # The basename call is just for diridx==0 (topdirs). Remove it if
@@ -433,29 +495,23 @@ class Folders(object):
                 continue
             thisdiridx = ids[0]
             thisdocidx = ids[1]
-            if thisdocidx >= 0:
-                doc = self._rcldocs[thisdocidx]
-            else:
-                # uplog("No doc for %s" % pid)
-                doc = None
-            
             if thisdiridx >= 0:
                 # Skip empty directories
                 if len(self._dirvec[thisdiridx]) == 1:
                     continue
                 id = self._idprefix + '$d' + str(thisdiridx)
-                if doc and doc.albumarturi:
-                    arturi = doc.albumarturi
-                else:
-                    arturi = self._arturifordir(thisdiridx)
-                entries.append(rcldirentry(id, pid, os.path.basename(nm),
+                arturi = self._arturifordir(thisdiridx)
+                entries.append(direntry(id, pid, os.path.basename(nm),
                                            arturi=arturi))
             else:
                 # Not a directory. docidx had better been set
                 if thisdocidx == -1:
                     uplog("folders:docidx -1 for non-dir entry %s"%nm)
                     continue
-                doc = self._rcldocs[thisdocidx]
+                if thisdocidx < len(self._rcldocs):
+                    doc = self._rcldocs[thisdocidx]
+                else:
+                    doc = self._moredocs[thisdocidx - len(self._rcldocs)]
                 id = self._idprefix + '$i' + str(thisdocidx)
                 e = rcldoctoentry(id, pid, self._httphp, self._pprefix, doc)
                 if e:
@@ -466,7 +522,7 @@ class Folders(object):
         # Add "Browse subtree by tags" entry
         if pid != self._idprefix and self._enabletags:
             id = pid + '$tagview.0'
-            entries.insert(0, rcldirentry(id, pid, ">> Tag View"))
+            entries.insert(0, direntry(id, pid, ">> Tag View"))
         return entries
 
     # Return path for objid, which has to be a container.This is good old
@@ -516,15 +572,22 @@ class Folders(object):
         return path
 
 
-    # Compute object id for doc out of recoll search. Not used at the moment.
-    # and _xid2idx is not built
+    # Compute object id for doc out of recoll search. Not used at the
+    # moment, and _xid2idx is not built.
     def _objidforxdocid(self, doc):
-        if doc.xdocid not in self._xid2idx:
+        if doc["xdocid"] not in self._xid2idx:
             return None
-        return self._idprefix + '$i' + str(self._xid2idx[doc.xdocid])
+        return self._idprefix + '$i' + str(self._xid2idx[doc["xdocid"]])
 
 
+    # Given a doc, we walk its url down from the part in root to find
+    # its directory entry, and return the _dirvec and _rcldocs indices
+    # it holds, either of which can be -1
     def _stat(self, doc):
+        # _pathbeyond... returns the _dirvec index of the root entry
+        # we start from (root is special), and the split rest of path.
+        # That is if the doc url has /av/mp3/classique/bach/ and the root entry is /av/mp3,
+        # we get the _dirvec entry index for /av/mp3 and [classique, bach]
         fathidx, pathl = self._pathbeyondtopdirs(doc)
         if not fathidx:
             return -1,-1
@@ -546,10 +609,13 @@ class Folders(object):
 
 
     def objidfordoc(self, doc):
-        if doc.mtype == 'inode/directory':
+        id=None
+        if doc["mtype"] == 'inode/directory':
             id = self._objidforpath(doc)
-        else:
-            id = self._objidforxdocid(doc)
+        #else:
+        #    # Note: not currently doing anything, see method comments
+        #    id = self._objidforxdocid(doc)
+
         if not id:
             id = self._idprefix + '$' + 'seeyoulater'
         return id
