@@ -28,6 +28,7 @@
 #include <utility>
 #include <vector>
 #include <sstream>
+#include <memory>
 
 #include "conftree.h"
 #include "main.hxx"
@@ -61,28 +62,36 @@ static const map<string, string> idmap {
     {"qobuz.com", "qobuz"}
 };
 
-
 // We might want to derive this into ServiceCredsQobuz,
 // ServiceCredsTidal, there is a lot in common and a few diffs.
 struct ServiceCreds {
+    ServiceCreds(const ServiceCreds&) = delete;
+    ServiceCreds &operator=(const ServiceCreds &) = delete;
     ServiceCreds() {}
     ServiceCreds(const string& inm, const string& u, const string& p,
                  const string& ep)
         : servicename(inm), user(u), password(p), encryptedpass(ep) {
 
         if (servicename == "qobuz") {
-            // The appid used by the Qobuz python module. Has to be
-            // consistent with the token obtained by the same, so we
-            // return it (by convention, as seen in wiresharking
-            // kazoo) in the data field. We could and do obtain the
-            // appid from the module, but kazoo apparently wants it
-            // before we login, so just hard-code it for now.  The
-            // Python code uses the value from XBMC,285473059,
-            // ohplayer uses 854233864
-            g_config->get("qobuzappid", data);
-            if (data.empty()) {
-                data = "285473059";
+            // The CP will want an appid in the 'Data' parameter out in the first actGet, so we need
+            // to retrieve it at once. We always call the Python code, which will retrieve the value
+            // using the method appropriate to the situation (config or dynamic).
+            if (!maybeStartCmd()) {
+                LOGERR("OHCreds: could not start Qobuz auxiliary process");
+                data = "0";
+                return;
             }
+            std::unordered_map<std::string, std::string> res;
+            auto toknm = "appid";
+            cmd->callproc("getappid", {}, res);
+            auto it = res.find(toknm);
+            if (it == res.end()) {
+                LOGERR("ServiceCreds::init: no " << toknm << " value in getappid call result\n");
+                return;
+            }
+            LOGINF("ServiceCreds: Got Qobuz appid [" << it->second << "] from plugin\n");
+            data = servicedata[toknm] = it->second;
+
         } else if (servicename == "tidal") {
             g_config->get("tidalapitoken", data);
             if (data.empty()) {
@@ -98,15 +107,21 @@ struct ServiceCreds {
         delete cmd;
     }
 
-    // We need a Python helper to perform the login. That's the media
-    // server gateway module, from which we only use a separate method
-    // which logs-in and returns the auth data (token, etc.)
+    void clearUserData() {
+        user.clear();
+        password.clear();
+        encryptedpass.clear();
+    }
+
+    // We need a Python helper to perform the login. That's the media server gateway module, from
+    // which we only specific methods for getting the app id and performing a login.
     bool maybeStartCmd() {
         LOGDEB1("ServiceCreds: " << servicename << " maybeStartCmd()\n");
         if (nullptr == cmd) {
             cmd = new CmdTalk(30);
         }
         if (cmd->running()) {
+            LOGDEB1("ServiceCreds: " << servicename << " already running\n");
             return true;
         }
         LOGDEB("ServiceCreds: " << servicename << " starting cmd\n");
@@ -133,8 +148,7 @@ struct ServiceCreds {
             return string();
         }
         unordered_map<string,string> res;
-        if (!cmd->callproc("login", {{"user", user},
-                    {"password", password}}, res)) {
+        if (!cmd->callproc("login", {{"user", user}, {"password", password}}, res)) {
             LOGERR("ServiceCreds::login: slave failure. Service " <<
                    servicename << " user " << user << endl);
             return string();
@@ -191,7 +205,7 @@ struct ServiceCreds {
     string password;
     string encryptedpass;
     bool enabled{true};
-    CmdTalk *cmd{0};
+    CmdTalk *cmd{nullptr};
     // Things we obtain from the module and send to the CP
     unordered_map<string,string> servicedata;
 
@@ -241,6 +255,26 @@ public:
         tryLoad();
     }
 
+    int insertCreds(const std::string& in_Id, const std::string& in_UserName,
+                    const string& plainpass, const string& in_Password) {
+        const auto it1 = idmap.find(in_Id);
+        if (it1 == idmap.end()) {
+            LOGERR("OHCredentials::actSet: bad service id [" << in_Id <<"]\n");
+            return 800;
+        }
+        string shortid = it1->second;
+        auto it = creds.find(in_Id);
+        if (it == creds.end()) {
+            creds[in_Id] = std::unique_ptr<ServiceCreds>(
+                new ServiceCreds(shortid, in_UserName, plainpass, in_Password));
+        } else if (!in_UserName.empty()) {
+            it->second->user = in_UserName;
+            it->second->password = plainpass;
+            it->second->encryptedpass = in_Password;
+        }
+        return 0;
+    }
+
     bool decrypt(const string& in, string& out) {
         vector<string> acmd{opensslcmd, "pkeyutl", "-inkey",
                 keyfile, "-pkeyopt", "rsa_padding_mode:oaep", "-decrypt"};
@@ -258,7 +292,7 @@ public:
         if (it == creds.end()) {
             return false;
         }
-        it->second.enabled = enabled;
+        it->second->enabled = enabled;
         return true;
     }
 
@@ -293,11 +327,9 @@ public:
             string data;
             ConfSimple credsconf(data);
             saveToConfTree(credsconf);
-            LockableShmSeg seg(ohcreds_segpath, ohcreds_segid, ohcreds_segsize,
-                               true);
+            LockableShmSeg seg(ohcreds_segpath, ohcreds_segid, ohcreds_segsize, true);
             if (!seg.ok()) {
-                LOGERR("OHCredentials: shared memory segment allocate/attach "
-                       "failed\n");
+                LOGERR("OHCredentials: shared memory segment allocate/attach failed\n");
                 return false;
             }
             LockableShmSeg::Accessor access(seg);
@@ -324,12 +356,12 @@ public:
     void saveToConfTree(ConfSimple& credsconf) {
         credsconf.clear();
         for (const auto& cred : creds) {
-            const string& shortid = cred.second.servicename;
-            credsconf.set(shortid + "user", cred.second.user);
-            credsconf.set(shortid + "pass", cred.second.password);
+            const string& shortid = cred.second->servicename;
+            credsconf.set(shortid + "user", cred.second->user);
+            credsconf.set(shortid + "pass", cred.second->password);
             // Saving the encrypted version is redundant, but it
             // avoids having to run encrypt on load.
-            credsconf.set(shortid + "epass", cred.second.encryptedpass);
+            credsconf.set(shortid + "epass", cred.second->encryptedpass);
         }
     }
 
@@ -354,7 +386,8 @@ public:
                 credsconf.get(shortid + "pass", pass) && 
                 credsconf.get(shortid + "epass", epass)) {
                 LOGDEB("OHCreds: using saved creds for " << id << endl);
-                creds[id] = ServiceCreds(shortid, user, pass, epass);
+                creds[id] =
+                    std::unique_ptr<ServiceCreds>(new ServiceCreds(shortid, user, pass, epass));
             }
         }
     }
@@ -364,7 +397,7 @@ public:
     string keyfile;
     string pubkey;
     int seq{1};
-    map<string, ServiceCreds> creds;
+    map<string, std::unique_ptr<ServiceCreds>> creds;
 };
 
 
@@ -444,24 +477,15 @@ int OHCredentials::actSet(const SoapIncoming& sc, SoapOutgoing& data)
     LOGDEB("OHCredentials::actSet: " << " Id " << in_Id << " UserName " <<
            in_UserName << " Password " << in_Password << endl);
 
-    const auto it1 = idmap.find(in_Id);
-    if (it1 == idmap.end()) {
-        LOGERR("OHCredentials::actSet: bad service id [" << in_Id <<"]\n");
-        return 800;
-    }
-    string servicename = it1->second;
     string cpass = base64_decode(in_Password);
     string plainpass;
     if (!m->decrypt(cpass, plainpass)) {
         LOGERR("OHCredentials::actSet: could not decrypt\n");
         return UPNP_E_INVALID_PARAM;
     }
-    auto it = m->creds.find(in_Id);
-    if (it == m->creds.end() || it->second.user != in_UserName ||
-        it->second.password != plainpass ||
-        it->second.encryptedpass != in_Password) {
-        m->creds[in_Id] = ServiceCreds(servicename, in_UserName, plainpass,
-                                       in_Password);
+    int ret;
+    if ((ret = m->insertCreds(in_Id, in_UserName, plainpass, in_Password))) {
+        return ret;
     }
     m->seq++;
     m->save();
@@ -489,14 +513,15 @@ int OHCredentials::actLogin(const SoapIncoming& sc, SoapOutgoing& data)
         LOGERR("OHCredentials::Login: Id " << in_Id << " not found\n");
         return 800;
     }
-    string token = it->second.login();
-    LOGDEB("OHCredentials::Login: got token " << token << endl);
+    string token = it->second->login();
+    LOGDEB("OHCredentials::Login: got token [" << token << "]\n");
     data.addarg("Token", token);
 
     // If login failed, erase the probably incorrect data from memory
     // and disk.
     if (token.empty()) {
-        m->creds.erase(in_Id);
+        LOGDEB("OHCredentials::Login: login failed\n");
+        it->second->clearUserData();
         m->save();
     }
 
@@ -528,8 +553,8 @@ int OHCredentials::actReLogin(const SoapIncoming& sc, SoapOutgoing& data)
         LOGERR("OHCredentials::Login: Id " << in_Id << " not found\n");
         return 800;
     }
-    it->second.logout();
-    string token = it->second.login();
+    it->second->logout();
+    string token = it->second->login();
     if (token.empty()) {
         return 801;
     }
@@ -597,36 +622,25 @@ int OHCredentials::actGet(const SoapIncoming& sc, SoapOutgoing& data)
 
     LOGDEB("OHCredentials::actGet: " << " Id " << in_Id << endl);
 
+    // Does nothing if the creds are already there. Else will create an object which may perform
+    // some init, like retrieving an appid
+    m->insertCreds(in_Id, "", "", "");
+
     auto it = m->creds.find(in_Id);
-    ServiceCreds emptycreds;
-    ServiceCreds *credsp(&emptycreds);
-    if (it != m->creds.end()) {
-        credsp = &(it->second);
-    } else {
-        if (in_Id == "qobuz") {
-            g_config->get("qobuzappid", emptycreds.data);
-            if (emptycreds.data.empty()) {
-                emptycreds.data = "285473059";
-            }
-        } else if (in_Id == "tidal") {
-            g_config->get("tidalapitoken", emptycreds.data);
-            if (emptycreds.data.empty()) {
-                emptycreds.data = "pl4Vc0hemlAXD0mN";
-            }
-        }
-        LOGDEB("OHCredentials::actGet: nothing found for " << in_Id << endl);
+    if (it == m->creds.end()) {
+        LOGERR("OHCredentials::actGet: Id " << in_Id << " not found or insert failed\n");
+        return UPNP_E_INVALID_PARAM;
     }
-    LOGDEB("OHCredentials::actGet: data for " << in_Id << " " <<
-           credsp->str() << endl);
-    data.addarg("UserName", credsp->user);
+    LOGDEB("OHCredentials::actGet: data for " << in_Id << " " << it->second->str() << endl);
+    data.addarg("UserName", it->second->user);
     // Encrypted password !
-    data.addarg("Password", credsp->encryptedpass);
+    data.addarg("Password", it->second->encryptedpass);
     // In theory enabled is set in response to setEnabled() or
     // set(). In practise, if it is not set, we don't get to the qobuz
     // settings screen in kazoo.
-    data.addarg("Enabled", credsp->enabled ? "1" : "1");
-    data.addarg("Status", credsp->status);
-    data.addarg("Data", credsp->data);
+    data.addarg("Enabled", it->second->enabled ? "1" : "1");
+    data.addarg("Status", it->second->status);
+    data.addarg("Data", it->second->data);
     return UPNP_E_SUCCESS;
 }
 
