@@ -60,6 +60,10 @@ static string find_script(const string& icmd)
     if (path_isabsolute(icmd))
         return icmd;
 
+    std::string scriptsdir;
+    if (!g_config->get("radioscripts", scriptsdir)) {
+        scriptsdir = path_cat(g_datadir, "radio_scripts");
+    }
     // Append the radio scripts dir to the PATH. Put at the end so
     // that the user can easily override a script by putting the
     // modified version in the PATH env variable
@@ -67,7 +71,7 @@ static string find_script(const string& icmd)
     if (!cp) //??
         cp = "";
     string PATH(cp);
-    PATH = PATH + path_PATHsep() + path_cat(g_datadir, "radio_scripts");
+    PATH = PATH + path_PATHsep() + scriptsdir;
     string cmd;
     if (ExecCmd::which(icmd, cmd, PATH.c_str())) {
         return cmd;
@@ -91,9 +95,15 @@ struct RadioMeta {
         }
         preferScript = stringToBool(ps);
     }
+
+    // The static title, URI and icon (from the radios configuration or
+    // setChannel) are always used in the ohradio interface. The
+    // dynamic currently playing title and metadata are only sent to
+    // OHInfo and obtainable from there.
     string title;
-    // Static playlist URI (from config)
     string uri;
+    string artUri;
+    
     // Script to retrieve current art
     vector<string> artScript; 
     // Script to retrieve all metadata
@@ -102,7 +112,6 @@ struct RadioMeta {
     // uri, which will normally be empty if the metascript is used for
     // audio).
     string currentAudioUri;
-    string artUri;
     // Keep values from script over mpd's (from icy)
     bool preferScript{true};
     // Time after which we should re-fire the metadata script
@@ -112,6 +121,10 @@ struct RadioMeta {
     string dynArtist;
 };
 
+
+// Our radio channels. The Id is an index is this. Channel 0 is
+// reserved for externally set Uri/Metadata from SetChannel. Our
+// preset channels (from the config) begin at 1
 static vector<RadioMeta> o_radios;
 
 OHRadio::OHRadio(UpMpd *dev, UpMpdOpenHome *udev)
@@ -128,7 +141,7 @@ OHRadio::OHRadio(UpMpd *dev, UpMpdOpenHome *udev)
         return;
     }
 
-    // Try to restore the current preset channel if this was memorized.
+    // Try to restore the channel if this was memorized.
     string refstr;
     if (g_state && g_state->get(cstr_sturlkey, refstr)) {
         for (unsigned int i = 0; i < o_radios.size(); i++) {
@@ -211,7 +224,7 @@ static void getRadiosFromConf(ConfSimple* conf)
 
 bool OHRadio::readRadios()
 {
-    // Id 0 means no selection
+    // Id 0 means no selection /  externally set channel from setChannel()
     o_radios.push_back(RadioMeta("Unknown radio", "", "", "", "", ""));
     
     getRadiosFromConf(g_config);
@@ -335,6 +348,7 @@ void OHRadio::maybeExecMetaScript(RadioMeta& radio, MpdStatus &mpds)
 
 bool OHRadio::makestate(unordered_map<string, string>& st)
 {
+    LOGDEB1("OHRadio::makestate\n");
     st.clear();
 
     MpdStatus mpds = m_dev->getMpdStatus();
@@ -344,28 +358,36 @@ bool OHRadio::makestate(unordered_map<string, string>& st)
     makeIdArray(st["IdArray"]);
     st["ProtocolInfo"] = Protocolinfo::the()->gettext();
     st["TransportState"] =  mpdstatusToTransportState(mpds.state);
-    st["Uri"] = mpds.currentsong.rsrc.uri;
-
     st["Metadata"] =  "";
+    st["Uri"] = "";
 
     if (!m_active) {
         st["TransportState"] =  "Stopped";
-        st["Uri"] = "";
         return true;
     }
 
     if (m_id < 0 || m_id >= o_radios.size()) {
         // ??
-        LOGDEB("OHRadio::makestate: bad m_id " << m_id << endl);
+        LOGERR("OHRadio::makestate: bad m_id " << m_id << endl);
         return true;
     }
         
-    if (mpds.currentsong.album.empty()) {
-        mpds.currentsong.album = o_radios[m_id].title;
-    }
-
     RadioMeta& radio = o_radios[m_id];
 
+    // In any case, Uri and Metadata are fixed and come from the channel.
+    // The dynamic data is sent to ohinfo.
+    st["Metadata"] = metaForId(m_id);
+    // We used to set this to mpds.currentsong.rsrc.uri. From
+    // re-reading the spec, using the radio uri seems more appropriate
+    st["Uri"] = radio.uri.empty() ? radio.currentAudioUri:radio.uri;
+
+    /////////////
+    // Compute dynamic metadata for ohinfo
+
+    if (mpds.currentsong.album.empty()) {
+        mpds.currentsong.album = radio.title;
+    }
+
     // Some radios do not insert icy metadata in the stream, but rather
     // provide a script to retrieve it.
     bool nompddata = mpds.currentsong.title.empty() &&
@@ -376,7 +398,10 @@ bool OHRadio::makestate(unordered_map<string, string>& st)
         mpds.currentsong.title = radio.dynTitle;
         mpds.currentsong.artist = radio.dynArtist;
     }
-
+    if (mpds.currentsong.title.empty()) {
+        mpds.currentsong.title = radio.title;
+    }
+        
     // Some radios provide a url to the art for the current song. 
     // Execute script to retrieve it if the current title+artist changed
     if (radio.artScript.size()) {
@@ -396,22 +421,31 @@ bool OHRadio::makestate(unordered_map<string, string>& st)
     mpds.currentsong.artUri = radio.dynArtUri.empty() ? radio.artUri :
         radio.dynArtUri;
 
-    string meta = didlmake(mpds.currentsong);
-    st["Metadata"] =  meta;
-    m_udev->getohif()->setMetatext(meta);
-    return true;
-}
+    // Don't report the ever changing bitrate, this causes unnecessary
+    // events. CPs interested in bitrate changes can get them from the
+    // Info service Details state variable
+    if (st["TransportState"] != "Playing") {
+        m_udev->getohif()->setMetadata(st["Metadata"], "");
+    } else {
+        string metatext = didlmake(mpds.currentsong, true);
+        string metadata;
+        UpSong temp;
+        if (uMetaToUpSong(st["Metadata"], &temp) &&
+            !mpds.currentsong.artUri.empty()) {
+            temp.artUri = mpds.currentsong.artUri;
+            metadata = didlmake(temp, true);
+        } else {
+            metadata = st["Metadata"];
+        }
 
-void OHRadio::maybeWakeUp(bool ok)
-{
-    if (ok) {
-        onEvent(nullptr);
+        m_udev->getohif()->setMetadata(metadata, metatext);
     }
+    return true;
 }
 
 int OHRadio::setPlaying()
 {
-    if (m_id > o_radios.size()) {
+    if (m_id >= o_radios.size()) {
         LOGERR("OHRadio::setPlaying: called with bad id (" << m_id << ")\n");
         return UPNP_E_INTERNAL_ERROR;
     }
@@ -428,8 +462,8 @@ int OHRadio::setPlaying()
         // We count on the metascript to also return an audio URI,
         // which will be sent to MPD during makestate().
         radio.currentAudioUri.clear();
-        m_dev->getmpdcli()->clearQueue();
         m_playpending = true;
+        m_dev->getmpdcli()->clearQueue();
         return UPNP_E_SUCCESS;
     }
     
@@ -487,7 +521,7 @@ void OHRadio::setActive(bool onoff)
                 m_dev->getmpdcli()->restoreState(m_mpdsavedstate);
             }
         }
-        maybeWakeUp(true);
+        onEvent(nullptr);
     } else {
         m_dev->getmpdcli()->saveState(m_mpdsavedstate);
         m_dev->getmpdcli()->clearQueue();
@@ -498,7 +532,6 @@ void OHRadio::setActive(bool onoff)
 int OHRadio::iPlay()
 {
     int ret = setPlaying();
-    maybeWakeUp(ret == UPNP_E_SUCCESS);
     return ret;
 }
 
@@ -506,7 +539,7 @@ int OHRadio::play(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHRadio::play" << endl);
     if (!m_active && m_udev->getohpr()) {
-        m_udev->getohpr()->iSetSourceIndexByName("Radio");
+        m_udev->getohpr()->iSetSourceIndexByName(OHRadioSourceName);
     }
     return iPlay();
 }
@@ -516,7 +549,6 @@ int OHRadio::pause(const SoapIncoming& sc, SoapOutgoing& data)
     LOGDEB("OHRadio::pause" << endl);
     bool ok = m_dev->getmpdcli()->pause(true);
     m_playpending = false;
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -524,7 +556,6 @@ int OHRadio::iStop()
 {
     bool ok = m_dev->getmpdcli()->stop();
     m_playpending = false;
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 int OHRadio::stop(const SoapIncoming& sc, SoapOutgoing& data)
@@ -554,7 +585,6 @@ int OHRadio::setChannel(const SoapIncoming& sc, SoapOutgoing& data)
         uMetaToUpSong(metadata, &ups);
         o_radios[0].title = ups.album + " " + ups.title;
     }
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -581,7 +611,6 @@ int OHRadio::setId(const SoapIncoming& sc, SoapOutgoing& data)
         g_state->set(cstr_sturlkey, refstr);
     }
     
-    maybeWakeUp(true);
     return UPNP_E_SUCCESS;
 }
 
@@ -593,59 +622,28 @@ int OHRadio::id(const SoapIncoming& sc, SoapOutgoing& data)
     return UPNP_E_SUCCESS;
 }
 
-static string radioDidlMake(const string& title, const string& uri,
-                            const string& artUri)
-{
-    string out("<DIDL-Lite xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n"
-               "xmlns:upnp=\"urn:schemas-upnp-org:metadata-1-0/upnp/\"\n"
-               "xmlns=\"urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/\">\n"
-               "<item id=\"\" parentID=\"\" restricted=\"True\">\n"
-               "<dc:title>");
-    out += SoapHelp::xmlQuote(title);
-    out += "</dc:title>\n"
-        "<res protocolInfo=\"*:*:*:*\" bitrate=\"6000\">";
-    if (uri.empty()) {
-        // Kazoo absolutely does not want uri to be empty, else it
-        // does not display anything in the radio list (not even the
-        // entries with uris). So fill up with bogus value. This is
-        // not used anyway because setId/setPlaying use the value from
-        // the radio array or from the metascript
-        out += SoapHelp::xmlQuote("http://www.bogus.com/bogus.mp3");
-    } else {
-        out += SoapHelp::xmlQuote(uri);
-    }
-    out += "</res>\n";
-    if (!artUri.empty()) {
-        out +=  "<upnp:albumArtURI>";
-        out += SoapHelp::xmlQuote(artUri);
-        out += "</upnp:albumArtURI>\n";
-    }
-    out +=  "<upnp:class>object.item.audioItem</upnp:class>\n"
-        "</item>\n"
-        "</DIDL-Lite>\n";
-    return out;
-}
-
 // This is called from read, and readlist. Don't send current metadata
 // (including dynamic art and song title) for the current channel,
 // else the radio logo AND name are replaced by the song's in channel
-// selection interfaces. Only send the song metadata with
-// OHRadio::Channel and Info:Metatext
+// selection interfaces. Only send the song metadata from the Info
+// service metatext variable
 string OHRadio::metaForId(unsigned int id)
 {
     LOGDEB1("OHRadio::metaForId: id " << id << " m_id " << m_id << endl);
-    string meta;
-    if (id >= 0 && id  < o_radios.size()) {
-        if (false && id == m_id) {
-            LOGDEB1("OHRadio::metaForId: using Metatext\n");
-            meta = m_state["Metadata"];
-        } else {
-            LOGDEB1("OHRadio::metaForId: using list data\n");
-            meta = radioDidlMake(o_radios[id].title, o_radios[id].uri, 
-                                 o_radios[id].artUri);
-        }
+    if (id  >= o_radios.size()) {
+        return string();
     }
-    return meta;
+    UpSong usong;
+    usong.title = o_radios[id].title;
+    // Kazoo absolutely does not want uri to be empty, else it
+    // does not display anything in the radio list (not even the
+    // entries with uris). So fill up with bogus value. This is
+    // not used anyway because setId/setPlaying use the value from
+    // the radio array or from the metascript
+    usong.rsrc.uri = o_radios[id].uri.empty() ?
+        "http://www.bogus.com/bogus.mp3" : o_radios[id].uri;
+    usong.artUri = o_radios[id].artUri;
+    return didlmake(usong);
 }
 
 // Report the metadata for a given channel id.
@@ -679,7 +677,6 @@ int OHRadio::ohread(const SoapIncoming& sc, SoapOutgoing& data)
 int OHRadio::readList(const SoapIncoming& sc, SoapOutgoing& data)
 {
     string sids;
-    UpSong song;
 
     bool ok = sc.get("IdList", &sids);
     LOGDEB("OHRadio::readList: [" << sids << "]" << endl);
@@ -688,15 +685,15 @@ int OHRadio::readList(const SoapIncoming& sc, SoapOutgoing& data)
     string out("<ChannelList>");
     if (ok) {
         stringToTokens(sids, ids);
-        for (auto it = ids.begin(); it != ids.end(); it++) {
-            int id = atoi(it->c_str());
+        for (auto strid : ids) {
+            int id = atoi(strid.c_str());
             if (id <= 0 || id >= int(o_radios.size())) {
                 LOGDEB("OHRadio::readlist: bad id " << id << endl);
                 continue;
             }
             string meta = metaForId(id);
             out += "<Entry><Id>";
-            out += *it;
+            out += strid;
             out += "</Id><Metadata>";
             out += SoapHelp::xmlQuote(meta);
             out += "</Metadata></Entry>";
@@ -729,7 +726,6 @@ int OHRadio::seekSecondAbsolute(const SoapIncoming& sc, SoapOutgoing& data)
     bool ok = sc.get("Value", &seconds);
     if (ok) {
         ok = m_dev->getmpdcli()->seek(seconds);
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -749,7 +745,6 @@ int OHRadio::seekSecondRelative(const SoapIncoming& sc, SoapOutgoing& data)
         } else {
             ok = false;
         }
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
