@@ -169,7 +169,7 @@ static void didlPrintResource(ostringstream& ss, const UpSong::Res& res) {
     ss << ">" << SoapHelp::xmlQuote(res.uri) << "</res>";
 }
 
-string UpSong::didl() const
+string UpSong::didl(bool noresource) const
 {
     ostringstream ss;
     string typetag;
@@ -204,9 +204,11 @@ string UpSong::didl() const
         UPNPXMLD(upnpClass, upnp:class, "object.item.audioItem.musicTrack");
         UPNPXML(album, upnp:album);
         UPNPXML(tracknum, upnp:originalTrackNumber);
-        didlPrintResource(ss, rsrc);
-        for (const auto& res : resources) {
-            didlPrintResource(ss, res);
+        if (!noresource) {
+            didlPrintResource(ss, rsrc);
+            for (const auto& res : resources) {
+                didlPrintResource(ss, res);
+            }
         }
     }
     UPNPXML(genre, upnp:genre);
@@ -241,9 +243,9 @@ string wrapDIDL(const std::string& data)
     return headDIDL() + data + tailDIDL();
 }
 
-string didlmake(const UpSong& song)
+string didlmake(const UpSong& song, bool noresource)
 {
-    return wrapDIDL(song.didl());
+    return wrapDIDL(song.didl(noresource));
 }
 
 bool dirObjToUpSong(const UPnPDirObject& dobj, UpSong *ups)
